@@ -399,7 +399,6 @@ async def fr_create_project(name: str, description: Optional[str] = None) -> Dic
         return create_error_response(f"Failed to create project: {str(e)}")
 
 
-@mcp.tool()
 @performance_monitor("fr_get_project")
 async def fr_get_project(project_identifier: Optional[Union[int, str]] = None) -> Dict[str, Any]:
     """Get a project from Freshrelease by ID or key.
@@ -986,6 +985,9 @@ async def fr_filter_tasks(
 ) -> Any:
     """Filter tasks/issues using various criteria with automatic name-to-ID resolution and custom field detection.
 
+    This function supports both individual field parameters and query-based filtering with comprehensive
+    name-to-ID resolution for all field types including custom fields.
+
     Args:
         project_identifier: Project ID or key (optional, uses FRESHRELEASE_PROJECT_KEY if not provided)
         query: Filter query in JSON string or comma-separated format (optional)
@@ -1025,14 +1027,24 @@ async def fr_filter_tasks(
         # Using issue keys for parent/epic
         fr_filter_tasks(parent_id="PROJ-123", epic_id="PROJ-456")
         
-        # Using query format with names
-        fr_filter_tasks(query="owner_id:John Doe,status_id:In Progress,cf_custom_field:value")
+        # Using query format with names and custom fields
+        fr_filter_tasks(query="owner_id:kalidass,theme:ITPM")
+        fr_filter_tasks(query="owner_id:John Doe,status_id:In Progress,cf_theme:ITPM")
         
-        # JSON format
-        fr_filter_tasks(query='{"owner_id":"John Doe","status_id":"In Progress"}', query_format="json")
+        # JSON format with custom fields
+        fr_filter_tasks(query='{"owner_id":"John Doe","status_id":"In Progress","theme":"ITPM"}', query_format="json")
+        
+        # Mixed ID and name filtering
+        fr_filter_tasks(owner_id=123, status_id="In Progress", sprint_id="Sprint 1")
         
         # Get all tasks (no filter)
         fr_filter_tasks()
+        
+    Note:
+        - All field names support both human-readable names and IDs
+        - Custom fields are automatically detected and prefixed with 'cf_'
+        - Name-to-ID resolution works for: owner_id, status_id, issue_type_id, sprint_id, release_id, sub_project_id
+        - Custom field values are also resolved to IDs when possible
     """
     try:
         # Validate environment variables
@@ -1066,12 +1078,53 @@ async def fr_filter_tasks(
         # Filter out None values
         field_params = {k: v for k, v in field_params.items() if v is not None}
 
-        # Build the final query
-        final_query = field_params
-        
-        # Make the API request
+        # Handle query parameter if provided
+        if query:
+            async with httpx.AsyncClient() as client:
+                # Get custom fields for the project to process query properly
+                custom_fields = await get_project_custom_fields(client, base_url, project_id, headers)
+                
+                # Parse query based on format
+                if query_format == "json":
+                    if isinstance(query, str):
+                        import json
+                        query_dict = json.loads(query)
+                    else:
+                        query_dict = query
+                    query_pairs = list(query_dict.items())
+                else:
+                    # Comma-separated format
+                    processed_query_str = process_query_with_custom_fields(query, custom_fields)
+                    query_pairs = parse_query_string(processed_query_str)
+                
+                # Resolve all field names and values to IDs
+                resolved_query = await _resolve_query_fields(
+                    query_pairs, project_id, client, base_url, headers, custom_fields
+                )
+                
+                # Make API request with resolved parameters
+                url = f"{base_url}/{project_id}/issues/filter"
+                result = await make_api_request("GET", url, headers, params=resolved_query)
+                return result
+
+        # Resolve names to IDs for individual field parameters
+        resolved_params = {}
+        if field_params:
+            async with httpx.AsyncClient() as client:
+                # Get custom fields for individual parameter processing
+                custom_fields = await get_project_custom_fields(client, base_url, project_id, headers)
+                
+                # Convert field_params to query_pairs format for consistent processing
+                query_pairs = list(field_params.items())
+                
+                # Use the same resolution logic as query processing
+                resolved_params = await _resolve_query_fields(
+                    query_pairs, project_id, client, base_url, headers, custom_fields
+                )
+
+        # Make the API request with individual parameters 
         url = f"{base_url}/{project_id}/issues/filter"
-        params = {"query": ",".join([f"{k}:{v}" for k, v in final_query.items()])}
+        params = resolved_params if resolved_params else {}
         
         result = await make_api_request("GET", url, headers, params=params)
         
@@ -1191,7 +1244,6 @@ async def fr_clear_filter_cache() -> Any:
         return create_error_response(f"Failed to clear cache: {str(e)}")
 
 
-@mcp.tool()
 async def fr_clear_lookup_cache() -> Any:
     """Clear the lookup cache for sprints, releases, tags, and subprojects.
     
@@ -1207,8 +1259,6 @@ async def fr_clear_lookup_cache() -> Any:
     except Exception as e:
         return create_error_response(f"Failed to clear lookup cache: {str(e)}")
 
-
-@mcp.tool()
 async def fr_clear_resolution_cache() -> Any:
     """Clear the resolution cache for name-to-ID lookups.
     
@@ -1500,6 +1550,59 @@ async def fr_filter_testcases(
         return create_error_response(f"Failed to filter test cases: {str(e)}")
 
 
+@mcp.tool()
+@performance_monitor("fr_get_issue_form_fields")
+async def fr_get_issue_form_fields(
+    project_identifier: Optional[Union[int, str]] = None,
+    issue_type_id: Optional[Union[int, str]] = None
+) -> Any:
+    """Get available fields and their possible values for issue creation and filtering.
+    
+    This tool returns the form fields that can be used for creating issues and filtering.
+    It shows both standard fields and custom fields available for the project.
+    
+    Args:
+        project_identifier: Project ID or key (optional, uses FRESHRELEASE_PROJECT_KEY if not provided)
+        issue_type_id: Issue type ID or name to get specific form fields (optional)
+    
+    Returns:
+        Form fields data with available fields and their possible values
+    """
+    try:
+        # Validate environment variables
+        env_data = validate_environment()
+        if "error" in env_data:
+            return env_data
+        
+        project_id = get_project_identifier(project_identifier)
+        base_url = env_data["base_url"]
+        headers = env_data["headers"]
+        client = get_http_client()
+
+        # Get issue form fields
+        url = f"{base_url}/{project_id}/issues/form"
+        
+        # Add issue type parameter if provided
+        params = {}
+        if issue_type_id:
+            # Resolve issue type name to ID if needed
+            if isinstance(issue_type_id, str) and not issue_type_id.isdigit():
+                issue_type_data = await _resolve_name_to_id_generic(
+                    issue_type_id, project_id, client, base_url, headers, "issue_types"
+                )
+                if isinstance(issue_type_data, dict) and "id" in issue_type_data:
+                    params["issue_type_id"] = issue_type_data["id"]
+                else:
+                    return create_error_response(f"Could not resolve issue type '{issue_type_id}' to ID")
+            else:
+                params["issue_type_id"] = issue_type_id
+        
+        return await make_api_request("GET", url, headers, client=client, params=params)
+
+    except Exception as e:
+        return create_error_response(f"Failed to get issue form fields: {str(e)}")
+
+
 @mcp.tool()
 @performance_monitor("fr_get_testcase_form_fields")
 async def fr_get_testcase_form_fields(
@@ -1536,6 +1639,69 @@ async def fr_get_testcase_form_fields(
 
 
 @mcp.tool()
+@performance_monitor("fr_get_all_issue_type_form_fields")
+async def fr_get_all_issue_type_form_fields(
+    project_identifier: Optional[Union[int, str]] = None
+) -> Any:
+    """Get form fields for all issue types in a project.
+    
+    This tool fetches form fields for each issue type in the project, allowing you to see
+    what fields are available for different types of issues (Bug, Task, Epic, etc.).
+    
+    Args:
+        project_identifier: Project ID or key (optional, uses FRESHRELEASE_PROJECT_KEY if not provided)
+    
+    Returns:
+        Dictionary with issue type names as keys and their form fields as values
+    """
+    try:
+        # Validate environment variables
+        env_data = validate_environment()
+        if "error" in env_data:
+            return env_data
+        
+        project_id = get_project_identifier(project_identifier)
+        base_url = env_data["base_url"]
+        headers = env_data["headers"]
+        client = get_http_client()
+
+        # First, get all issue types
+        issue_types_url = f"{base_url}/{project_id}/issue_types"
+        issue_types_data = await make_api_request("GET", issue_types_url, headers, client=client)
+        
+        if not isinstance(issue_types_data, list):
+            return create_error_response("Failed to fetch issue types", issue_types_data)
+        
+        # Get form fields for each issue type
+        form_fields_by_type = {}
+        
+        for issue_type in issue_types_data:
+            issue_type_id = issue_type.get("id")
+            issue_type_name = issue_type.get("name", f"Type_{issue_type_id}")
+            
+            if issue_type_id:
+                try:
+                    form_url = f"{base_url}/{project_id}/issues/form"
+                    form_data = await make_api_request(
+                        "GET", form_url, headers, client=client, 
+                        params={"issue_type_id": issue_type_id}
+                    )
+                    form_fields_by_type[issue_type_name] = form_data
+                except Exception as e:
+                    form_fields_by_type[issue_type_name] = {
+                        "error": f"Failed to fetch form fields: {str(e)}"
+                    }
+        
+        return {
+            "project_id": project_id,
+            "issue_types": form_fields_by_type,
+            "total_issue_types": len(issue_types_data)
+        }
+
+    except Exception as e:
+        return create_error_response(f"Failed to get all issue type form fields: {str(e)}")
+
+
 async def fr_clear_testcase_form_cache() -> Any:
     """Clear the test case form cache.
     
@@ -1577,7 +1743,6 @@ async def fr_clear_all_caches() -> Any:
         return create_error_response(f"Failed to clear caches: {str(e)}")
 
 
-@mcp.tool()
 async def fr_get_performance_stats() -> Dict[str, Any]:
     """Get performance statistics for all monitored functions.
     
@@ -1591,7 +1756,6 @@ async def fr_get_performance_stats() -> Dict[str, Any]:
         return create_error_response(f"Failed to get performance stats: {str(e)}")
 
 
-@mcp.tool()
 async def fr_clear_performance_stats() -> Dict[str, Any]:
     """Clear performance statistics.
     
@@ -1604,8 +1768,6 @@ async def fr_clear_performance_stats() -> Dict[str, Any]:
     except Exception as e:
         return create_error_response(f"Failed to clear performance stats: {str(e)}")
 
-
-@mcp.tool()
 async def fr_close_http_client() -> Dict[str, Any]:
     """Close the global HTTP client to free resources.
     
@@ -1789,6 +1951,142 @@ async def _resolve_name_to_id_generic(
     return item["id"]
 
 
+async def _resolve_user_name_to_id(
+    user_identifier: str,
+    project_id: Union[int, str],
+    client: httpx.AsyncClient,
+    base_url: str,
+    headers: Dict[str, str]
+) -> int:
+    """Resolve user name or email to user ID."""
+    # First try to find by exact name match
+    try:
+        url = f"{base_url}/{project_id}/users"
+        params = {"q": user_identifier}
+        response = await client.get(url, headers=headers, params=params)
+        response.raise_for_status()
+        users = response.json()
+        
+        if isinstance(users, list) and users:
+            # Look for exact name match first
+            for user in users:
+                if user.get("name", "").lower() == user_identifier.lower():
+                    return user["id"]
+                if user.get("email", "").lower() == user_identifier.lower():
+                    return user["id"]
+            
+            # If no exact match, return the first result
+            return users[0]["id"]
+        
+        raise ValueError(f"User '{user_identifier}' not found")
+    except Exception as e:
+        raise ValueError(f"Failed to resolve user '{user_identifier}': {str(e)}")
+
+
+async def _resolve_subproject_name_to_id(
+    sub_project_name: str,
+    project_id: Union[int, str]
+) -> int:
+    """Resolve sub-project name to ID using the utility function.
+    
+    This function integrates the new get_subproject_id_by_name utility
+    with the existing field resolution system for task filtering.
+    
+    Note: The project_id parameter is kept for compatibility with the field resolver
+    system, but the utility function uses the environment variable.
+    
+    Args:
+        sub_project_name: Name of the sub-project to resolve
+        project_id: Project ID or key (unused - kept for compatibility)
+        
+    Returns:
+        Sub-project ID as integer
+        
+    Raises:
+        ValueError: If sub-project name cannot be resolved with helpful error message
+    """
+    # Use the utility function for consistent sub-project resolution
+    result = await get_subproject_id_by_name(sub_project_name)
+    
+    if "error" in result:
+        # If there's an error, raise an exception with helpful info
+        available = ", ".join(result.get("available_sub_projects", []))
+        error_msg = result["error"]
+        if available:
+            error_msg += f". Available sub-projects: {available}"
+        raise ValueError(error_msg)
+    
+    return result["sub_project_id"]
+
+
+async def _resolve_query_fields(
+    query_pairs: List[tuple],
+    project_id: Union[int, str],
+    client: httpx.AsyncClient,
+    base_url: str,
+    headers: Dict[str, str],
+    custom_fields: List[Dict[str, Any]]
+) -> Dict[str, Any]:
+    """Resolve all query field names and values to their proper IDs.
+    
+    Handles:
+    - Standard fields (owner_id, status_id, issue_type_id, sprint_id, release_id, sub_project_id)
+    - Custom fields (with cf_ prefix)
+    - Name-to-ID resolution for all supported field types
+    """
+    resolved_query = {}
+    
+    # Field resolution mapping
+    field_resolvers = {
+        "owner_id": lambda value: _resolve_user_name_to_id(value, project_id, client, base_url, headers),
+        "status_id": lambda value: _resolve_name_to_id_generic(value, project_id, client, base_url, headers, "statuses"),
+        "issue_type_id": lambda value: _resolve_name_to_id_generic(value, project_id, client, base_url, headers, "issue_types"),
+        "sprint_id": lambda value: _resolve_name_to_id_generic(value, project_id, client, base_url, headers, "sprints"),
+        "release_id": lambda value: _resolve_name_to_id_generic(value, project_id, client, base_url, headers, "releases"),
+        "sub_project_id": lambda value: _resolve_subproject_name_to_id(value, project_id),
+    }
+    
+    for field_name, value in query_pairs:
+        try:
+            # Handle custom fields
+            if field_name.startswith("cf_") or is_custom_field(field_name, custom_fields):
+                # Ensure custom field has cf_ prefix
+                if not field_name.startswith("cf_"):
+                    field_name = f"cf_{field_name}"
+                
+                # For custom fields, try to resolve value to ID if it's a string
+                if isinstance(value, str):
+                    try:
+                        resolved_value = await _resolve_custom_field_value_optimized(
+                            field_name, value, project_id, client, base_url, headers
+                        )
+                        resolved_query[field_name] = resolved_value
+                    except Exception:
+                        # If custom field resolution fails, use original value
+                        resolved_query[field_name] = value
+                else:
+                    resolved_query[field_name] = value
+            
+            # Handle standard fields with name-to-ID resolution
+            elif field_name in field_resolvers and isinstance(value, str):
+                try:
+                    resolved_value = await field_resolvers[field_name](value)
+                    resolved_query[field_name] = resolved_value
+                except Exception:
+                    # If resolution fails, use original value
+                    resolved_query[field_name] = value
+            
+            # Handle other fields (pass through as-is)
+            else:
+                resolved_query[field_name] = value
+                
+        except Exception as e:
+            # If any error occurs, use original value
+            resolved_query[field_name] = value
+    
+    return resolved_query
+
+
 async def _resolve_custom_field_value_optimized(
     field_name: str,
     value: str,
@@ -1802,8 +2100,200 @@ async def _resolve_custom_field_value_optimized(
     return value
 
 
+async def get_subproject_id_by_name(
+    sub_project_name: str
+) -> Dict[str, Any]:
+    """Utility function to get sub-project ID and info by name.
+    
+    This utility function can be used by other MCP tools that need to resolve
+    sub-project names to IDs. It provides consistent error handling and caching.
+    
+    Usage Example in other functions:
+        subproject_result = await get_subproject_id_by_name("Frontend Development")
+        if "error" in subproject_result:
+            return subproject_result
+        sub_project_id = subproject_result["sub_project_id"]
+        sub_project_info = subproject_result["sub_project_info"]
+    
+    Args:
+        sub_project_name: Name of the sub-project to find (required)
+        
+    Returns:
+        Dictionary with sub_project_id, sub_project_info, or error
+        Format: {"sub_project_id": int, "sub_project_info": dict} or {"error": str, "available_sub_projects": list}
+    """
+    try:
+        project_id = get_project_identifier()
+        
+        if not sub_project_name:
+            return {"error": "sub_project_name is required"}
+        
+        headers = {
+            "Authorization": f"Token {FRESHRELEASE_API_KEY}",
+            "Content-Type": "application/json"
+        }
+        
+        client = get_http_client()
+        
+        # Get all sub-projects to find the ID by name
+        # Handle both project keys (like "FS", "PROJ") and project IDs (like 123)
+        sub_projects_url = f"https://{FRESHRELEASE_DOMAIN}/api/projects/{project_id}/sub_projects"
+        
+        logging.info(f"Fetching sub-projects from: {sub_projects_url}")
+        sub_projects_response = await client.get(sub_projects_url, headers=headers)
+        
+        if sub_projects_response.status_code != 200:
+            logging.error(f"Failed to fetch sub-projects: {sub_projects_response.status_code}")
+            return {
+                "error": f"Failed to fetch sub-projects: {sub_projects_response.status_code}",
+                "details": sub_projects_response.text
+            }
+        
+        sub_projects_data = sub_projects_response.json()
+        
+        # Debug: Log the actual response structure
+        logging.info(f"Sub-projects API response structure: {type(sub_projects_data)}")
+        logging.info(f"Sub-projects API response keys: {list(sub_projects_data.keys()) if isinstance(sub_projects_data, dict) else 'Not a dict'}")
+        
+        # Handle different possible response structures
+        if isinstance(sub_projects_data, list):
+            # Response is directly a list of sub-projects
+            sub_projects = sub_projects_data
+        elif isinstance(sub_projects_data, dict):
+            # Response is an object, try different possible keys
+            sub_projects = (sub_projects_data.get("sub_projects") or 
+                          sub_projects_data.get("subprojects") or 
+                          sub_projects_data.get("sub_project") or
+                          sub_projects_data.get("data") or
+                          sub_projects_data.get("results") or [])
+        else:
+            return {
+                "error": f"Unexpected response structure for sub_projects",
+                "response_type": str(type(sub_projects_data)),
+                "response_preview": str(sub_projects_data)[:500]  # First 500 chars for debugging
+            }
+        
+        # Validate we have a list to work with
+        if not isinstance(sub_projects, list):
+            return {
+                "error": f"Expected sub_projects to be a list, got {type(sub_projects)}",
+                "response_data": sub_projects_data,
+                "sub_projects_value": sub_projects
+            }
+        
+        # Find sub-project by name (case-insensitive)
+        for sub_project in sub_projects:
+            if sub_project.get("name", "").lower() == sub_project_name.lower():
+                return {
+                    "sub_project_id": sub_project.get("id"),
+                    "sub_project_info": sub_project
+                }
+        
+        # Sub-project not found
+        available_names = [sp.get("name", "") for sp in sub_projects]
+        return {
+            "error": f"Sub-project '{sub_project_name}' not found",
+            "available_sub_projects": available_names
+        }
+        
+    except Exception as e:
+        logging.error(f"Error getting sub-project ID by name: {str(e)}")
+        return {"error": f"Failed to get sub-project ID: {str(e)}"}
+
+
+@mcp.tool()
+@performance_monitor("fr_get_current_subproject_sprint")
+async def fr_get_current_subproject_sprint(
+    sub_project_name: str
+) -> Dict[str, Any]:
+    """Get the current active sprint for a sub-project by name.
+    
+    This function first resolves the sub-project name to ID, then fetches
+    the active sprints for that sub-project and returns the current one.
+    
+    Args:
+        sub_project_name: Name of the sub-project to get current sprint for (required)
+        
+    Returns:
+        Current active sprint data or error response
+        
+    Examples:
+        # Get current sprint for a sub-project (uses FRESHRELEASE_PROJECT_KEY)
+        fr_get_current_subproject_sprint(sub_project_name="Frontend Development")
+        
+        # Get current sprint for Backend API sub-project
+        fr_get_current_subproject_sprint(sub_project_name="Backend API")
+    """
+    try:
+        # Step 1: Get sub-project ID by name using utility function
+        subproject_result = await get_subproject_id_by_name(sub_project_name)
+        
+        if "error" in subproject_result:
+            return subproject_result
+        
+        sub_project_id = subproject_result["sub_project_id"]
+        sub_project_info = subproject_result["sub_project_info"]
+        
+        # Get project identifier (avoid redundant call since utility function already calls this)
+        project_id = get_project_identifier()
+        
+        headers = {
+            "Authorization": f"Token {FRESHRELEASE_API_KEY}",
+            "Content-Type": "application/json"
+        }
+        
+        client = get_http_client()
+        
+        # Step 2: Get active sprints for the sub-project
+        # Handle both project keys (like "FS", "PROJ") and project IDs (like 123)
+        # The sprints API can accept both project keys and IDs
+        sprints_url = f"https://{FRESHRELEASE_DOMAIN}/api/{project_id}/sprints"
+        sprints_params = {
+            "primary_workspace_id": sub_project_id,
+            "query_hash[0][condition]": "state",
+            "query_hash[0][operator]": "is_in", 
+            "query_hash[0][value]": "2"  # 2 = active state
+        }
+        
+        logging.info(f"Fetching active sprints from: {sprints_url}")
+        logging.info(f"Sprint params: {sprints_params}")
+        
+        sprints_response = await client.get(sprints_url, headers=headers, params=sprints_params)
+        
+        if sprints_response.status_code != 200:
+            logging.error(f"Failed to fetch sprints: {sprints_response.status_code}")
+            return {
+                "error": f"Failed to fetch sprints: {sprints_response.status_code}",
+                "details": sprints_response.text
+            }
+        
+        sprints_data = sprints_response.json()
+        sprints = sprints_data.get("sprints", [])
+        
+        if not sprints:
+            return {
+                "message": f"No active sprints found for sub-project '{sub_project_name}'",
+                "sub_project": sub_project_info,
+                "active_sprints": []
+            }
+        
+        # Return the first active sprint (current sprint)
+        current_sprint = sprints[0]
+        
+        return {
+            "current_sprint": current_sprint,
+            "sub_project": sub_project_info,
+            "total_active_sprints": len(sprints),
+            "all_active_sprints": sprints
+        }
+        
+    except Exception as e:
+        logging.error(f"Error getting current sub-project sprint: {str(e)}")
+        return {"error": f"Failed to get current sub-project sprint: {str(e)}"}
+
+
 def main():
-    logging.info("Starting Freshdesk MCP server")
+    logging.info("Starting Freshrelease MCP server")
     mcp.run(transport='stdio')
 
 if __name__ == "__main__":
